C                                                            <HTML><PRE>
C Subroutine file 'scronum' - dummy screen output subroutines
C
C Date: 1999, September 23
C Coded by: Ludek Klimes
C
C.......................................................................
C
C This file consists of the following external procedures:
C     SCRO1...Screen output subroutine called when starting the complete
C             ray tracing program, and when starting the computation of
C             a new elementary wave.
C             <A HREF="#SCRO1">SCRO1</A>
C     SCRO2...Screen output subroutine called when starting the complete
C             tracing of a new ray.
C             <A HREF="#SCRO2">SCRO2</A>
C     SCRO3...Screen output subroutine called with constant step STORE
C             of the independent variable along the ray, and at the
C             points of intersection with interfaces either before and
C             after the transformation.
C             <A HREF="#SCRO3">SCRO3</A>
C     SCRO4...Screen output subroutine called after termination of
C             tracing the ray.
C             <A HREF="#SCRO4">SCRO4</A>
C     SCRO5...Screen output subroutine called after termination of the
C             computation of an elementary wave, and when terminating
C             the complete ray tracing program.
C             <A HREF="#SCRO5">SCRO5</A>
C
C=======================================================================
C
C     <A NAME="SCRO1"></A>
C
      SUBROUTINE SCRO1(IWAVE)
      INTEGER IWAVE
C
C This screen output subroutine is called when starting the complete ray
C tracing program, and when starting the computation of a new elementary
C wave.
C
C Input:
C     IWAVE...Zero when starting the complete ray tracing program,
C             otherwise the index of the elementary wave which will be
C             computed (i.e. the output of the subroutine CODE1 from the
C             file 'code.for').
C
C No output.
C
C Date: 1999, September 23
C Coded by Ludek Klimes
C
C-----------------------------------------------------------------------
C
      INTEGER ISRC
      SAVE ISRC
      DATA ISRC/0/
C
C     Writing to the screen:
      IF(IWAVE.EQ.1) THEN
        ISRC=ISRC+1
      END IF
      WRITE(*,'(A,2(I4,A))') '+                           WAVE:',IWAVE,
     *                 '        SOURCE:',ISRC,'                        '
      RETURN
      END
C
C=======================================================================
C
C     <A NAME="SCRO2"></A>
C
      SUBROUTINE SCRO2(IRAY)
      INTEGER IRAY
C
C This screen output subroutine is called when starting the complete
C tracing of a new ray.
C
C Input:
C     IRAY... The index of the ray which will be computed (i.e. the
C             output of the subroutine RPAR2 from the file 'rpar.for').
C
C No output.
C
C Date: 1996, April 30
C Coded by Ludek Klimes
C
C-----------------------------------------------------------------------
C
C     Writing to the screen:
      WRITE(*,'(A,I8)')   '+TRACING RAY',IRAY
      RETURN
      END
C
C=======================================================================
C
C     <A NAME="SCRO3"></A>
C
      SUBROUTINE SCRO3(YL,Y,YY,IY)
      REAL YL(6),Y(35),YY(5)
      INTEGER IY(12)
C
C This screen output subroutine is called with constant step STORE of
C the independent variable along the ray, and at the points of
C intersection with interfaces either before and after the
C transformation.  It plots the part of the ray computed in the last
C step of the numerical integration.  It is called by the subroutine
C WRIT31.
C
C Input:
C     YL...   Array containing local quantities at the point of the ray.
C     Y...    Array containing basic quantities computed along the ray.
C     YY...   Array containing real auxiliary quantities computed along
C             the ray.
C     IY...   Array containing integer auxiliary quantities computed
C             along the ray.
C None of the input parameters are altered.
C
C No output.
C
C Date: 1989, December 4
C Coded by Ludek Klimes
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
C     <A NAME="SCRO4"></A>
C
      SUBROUTINE SCRO4(IRAY,YL,Y,YY,IY,IEND,ISHEET)
C
      INTEGER IRAY,IY(12),IEND,ISHEET
      REAL YL(6),Y(35),YY(5)
C
C This screen output subroutine is called after termination of tracing
C the ray.
C
C Input:
C     IRAY... The index of the ray which has been computed (i.e. the
C             output of the subroutine RPAR2 of the file 'rpar.for').
C     YL...   Array containing local quantities at the point of the ray.
C     Y...    Array containing basic quantities computed along the ray.
C     YY...   Array containing real auxiliary quantities computed along
C             the ray.
C     IY...   Array containing integer auxiliary quantities computed
C             along the ray.
C     IEND... Reason of the termination of the computation of a ray (see
C             C.R.T.5.4).  For a detailed description see subroutine RAY
C             (subroutine file 'ray.for').
C     ISHEET..Ray-history index.  The different ray histories are
C             consecutively indexed by positive integers 1,2,3,...
C             According to their appearance during ray tracing.
C             The ray histories are indexed independently within each
C             elementary wave.
C             The ray-history indices are complemented with sign:
C             Positive - successful ray (crossing reference surface),
C             negative - unsuccessful ray (terminating before crossing
C             reference surface).
C
C No output.
C
C Date: 1997, September 29
C Coded by Ludek Klimes
C
C-----------------------------------------------------------------------
C
C     Writing to the screen:
      WRITE(*,'(A)') '+AIMING     '
      RETURN
      END
C
C=======================================================================
C
C     <A NAME="SCRO5"></A>
C
      SUBROUTINE SCRO5(IWAVE)
      INTEGER IWAVE
C
C This screen output subroutine is called after termination of the
C computation of an elementary wave, and when terminating the complete
C ray tracing program.
C
C Input:
C     IWAVE...Zero when terminating the complete ray tracing program,
C             otherwise the index of the elementary wave which has been
C             computed (i.e. the output of the subroutine CODE1 from the
C             file 'code.for').
C
C No output.
C
C Date: 1997, September 29
C Coded by Ludek Klimes
C
C-----------------------------------------------------------------------
C
C     Writing to the screen:
      WRITE(*,'(A)') '+FINISHED   '
*     WRITE(*,'(A)')
      RETURN
      END
C
C=======================================================================
C                                                                 </PRE>
